#include "vex-encoder.hpp"

// VexEncoder* VexEncoder::lastEncoder = nullptr;

VexEncoder::VexEncoder(VexEncoderInit& encoderInit, VexEncoder* lastEncoder)
    : _i2c(encoderInit.i2c), _address(0), _position(0), _maxPosition(encoderInit.maxPosition) {
	uint8_t newAddress = VEX_STARTING_ADDRESS;

	if (lastEncoder != nullptr) {
		lastEncoder->unterminate();
		newAddress = lastEncoder->_address + 0x2;
	}

	setAddress(newAddress);
	zero();
}

float VexEncoder::getPosition() {
	updatePosition();
	return _position;
}

void VexEncoder::updatePosition() {
	_position = MOTOR_393_TORQUE_ROTATION / TICKS * getRawPosition();
}

float VexEncoder::getMaxPosition() {
	return _maxPosition;
}

uint8_t VexEncoder::getAddress() {
	return _address;
}

void VexEncoder::setAddress(uint8_t newAddress) {
	// Increment I2C by factor of 2 according to datasheet
	VexEncoder::_setAddress(newAddress, _i2c);
	_address = newAddress;
}

void VexEncoder::unterminate() {
	VexEncoder::_unterminate(_address, _i2c);
}

void VexEncoder::terminate() {
	VexEncoder::_terminate(_address, _i2c);
}

void VexEncoder::accessRegister(uint8_t reg) {
	VexEncoder::_accessRegister(reg, _address, _i2c);
}

void VexEncoder::zero() {
	accessRegister(I2C_ENCODER_ZERO_REGISTER);
}

void VexEncoder::monitorVexPosition() {}

long VexEncoder::getRawPosition() {
	uint8_t raw_position[4];
	accessRegister(I2C_ENCODER_POSITION_REGISTER);
	_i2c.blockingRead(_address, raw_position, sizeof(raw_position));

	long position = 0;
	position |= raw_position[0] << 8;
	position |= raw_position[1];
	position |= raw_position[2] << 24;
	position |= raw_position[3] << 16;

	// TODO - check for negative numbers (motor is reversed)
	return position;
}

// void VexEncoder::writeToVex(uint8_t reg, uint8_t address, I2c& i2c, uint8_t* bytes, uint8_t bufferSize) {
// 	// i2c.blockingWrite(address, &reg, bufferSize);
// }

void VexEncoder::_accessRegister(uint8_t reg, uint8_t address, I2c& i2c) {
	i2c.blockingWrite(address, &reg, 1);
}

void VexEncoder::_terminate(uint8_t address, I2c& i2c) {
	VexEncoder::_accessRegister(I2C_ENCODER_TERMINATE_REGISTER, address, i2c);
}

void VexEncoder::_unterminate(uint8_t address, I2c& i2c) {
	VexEncoder::_accessRegister(I2C_ENCODER_UNTERMINATE_REGISTER, address, i2c);
}

void VexEncoder::_setAddress(uint8_t newAddress, I2c& i2c) {
	// Increment I2C by factor of 2 according to datasheet
	uint8_t changeAddressBytes[2] = {I2C_ENCODER_ADDRESS_REGISTER, newAddress};
	i2c.blockingWrite(I2C_ENCODER_DEFAULT_ADDRESS, changeAddressBytes, sizeof(changeAddressBytes));
}

void VexEncoder::resetAllEncoders(uint8_t numberOfMotor, I2c& i2c) {
	const uint8_t address = 0x20;
	for (int i = 0; i < numberOfMotor; i++) {
		uint8_t resetBytes[3] = {I2C_ENCODER_RESET_ALL_DEVICES, 0xCA, 0x03};
		i2c.blockingWrite(0x00, resetBytes, sizeof(resetBytes));

		// Increment I2C by factor of 2 according to datasheet
		uint8_t changeAddressBytes[2] = {I2C_ENCODER_ADDRESS_REGISTER, address};
		i2c.blockingWrite(I2C_ENCODER_DEFAULT_ADDRESS, changeAddressBytes, sizeof(changeAddressBytes));
		VexEncoder::_unterminate(address, i2c);
	}

	uint8_t resetBytes[3] = {I2C_ENCODER_RESET_ALL_DEVICES, 0xCA, 0x03};
	i2c.blockingWrite(0x00, resetBytes, sizeof(resetBytes));
}
