#ifndef BRUSHED_MOTOR_H
#define BRUSHED_MOTOR_H

#include <stdlib.h>

#include <alternate-gpio-init.hpp>
#include <alternate-gpio.hpp>
#include <output-gpio-init.hpp>
#include <output-gpio.hpp>

#include "brushed-motor-init.hpp"
#include "motor-direction.hpp"
#include "pwm-timer.hpp"

using namespace stm32f072xb;

template <Port pwmPort, Port directionPort>
class BrushedMotor {
   private:
	PwmTimer _timer;
	AlternateGpio<pwmPort> _pwmOutputPin;
	OutputGpio<directionPort> _directionPin;

   public:
	BrushedMotor(BrushedMotorInit& motorInit, const uint16_t pwmResolution = 128, const uint16_t pwmFrequency = 1000)
	    : _timer(motorInit.timer, motorInit.OCC, pwmResolution, pwmFrequency),
	      _pwmOutputPin({motorInit.pwmPin, Pupdr::NONE, motorInit.alternateFunction}),
	      _directionPin({motorInit.directionPin, Pupdr::NONE}) {}

	void setSpeed(int16_t speed) {
		_timer.setDutyCycle(abs(speed));
		setDirection(speed >= 0 ? MotorDirection::CLOCK_WISE : MotorDirection::COUNTER_CLOCK_WISE);
	}

	void setDirection(MotorDirection direction) {
		if (direction == MotorDirection::CLOCK_WISE) {
			_directionPin.setPin(FlagStatus::SET);
		} else {
			_directionPin.setPin(FlagStatus::RESET);
		}
	}
};

#endif
